'''
The first step in making a new release of Brian is to run all the
tests to make sure that nothing has been broken by any new
changes, and that every object is still behaving as we say it
should.
'''

from brian import *
run_all_tests()
